/**
  ******************************************************************************
  * @file    STemWin/STemWin_HelloWorld/Src/main.c
  * @author  MCD Application Team
  * @version V1.0.4
  * @date    22-April-2016
  * @brief   This file provides main program functions
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2016 STMicroelectronics International N.V.
  * All rights reserved.</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted, provided that the following conditions are met:
  *
  * 1. Redistribution of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  * 3. Neither the name of STMicroelectronics nor the names of other
  *    contributors to this software may be used to endorse or promote products
  *    derived from this software without specific written permission.
  * 4. This software, including modifications and/or derivative works of this
  *    software, must execute solely and exclusively on microcontroller or
  *    microprocessor devices manufactured by or for STMicroelectronics.
  * 5. Redistribution and use of this software other than as permitted under
  *    this license is void and will automatically terminate your rights under
  *    this license.
  *
  * THIS SOFTWARE IS PROVIDED BY STMICROELECTRONICS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
  * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT
  * SHALL STMICROELECTRONICS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "WM.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint8_t GUI_Initialized = 0;
TIM_HandleTypeDef TimHandle;
uint32_t uwPrescalerValue = 0;

/* Private function prototypes -----------------------------------------------*/
static void BSP_Config(void);
static void SystemClock_Config(void);
void BSP_Pointer_Update(void);
void BSP_Background(void);

extern void MainTask(void);
extern unsigned char ts_en;

// static void delay_ms(int t);


/* UART handler declaration */
UART_HandleTypeDef UartHandle;
/* Private function prototypes -----------------------------------------------*/
#ifdef __GNUC__
/* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
   set to 'Yes') calls __io_putchar() */
#define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
#define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */


//********************************************
#include "btn.h"

#define BTN_CNT 2
#define BTN_RST 0x00000001
#define BTN_IMG 0x00000002

btn_area_t btns[BTN_CNT];

uint32_t  btn_state = 0;
uint32_t  btn_state_prev = 0;
uint32_t  btn_active = 0;
uint32_t  btn_press = 0;
uint32_t  btn_release = 0;

//********************************************

//********************************************
#include "sldr.h"
#define SLDR_V_CNT  2
#define SLDR_V_AG  0x00000001
#define SLDR_V_DG  0x00000002

sldr_area_t sldrs_v[SLDR_V_CNT];

uint32_t  sldr_v_state = 0;


//********************************************
#define IMG_X_SIZE    213 //182
#define IMG_Y_SIZE    170 //146
#define IMG_PIX_BYTES 3
// uint8_t img[IMG_SIZE * IMG_PIX_BYTES];


#define MSG_MAX_LEN ((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) + 6)
#define MSG_START '#'
#define MSG_STOP  '*'

uint8_t   msg_ch;
unsigned char  msg[MSG_MAX_LEN];
// typedef uint8_t msg_t[MSG_MAX_LEN];

// msg_t *msg = (msg_t *)SDRAM_USR_ADDR;

// static uint8_t  *msg = (uint8_t *)SDRAM_USR_ADDR;
// uint16_t  msg_len;
// uint8_t   msg_active;
uint8_t   msg_parse;


GUI_CONST_STORAGE GUI_BITMAP _bm_img= {
  IMG_X_SIZE,                   /* XSize */
  IMG_Y_SIZE,                   /* YSize */
  IMG_X_SIZE * IMG_PIX_BYTES,   /* BytesPerLine */
  IMG_PIX_BYTES * 8,            /* BitsPerPixel */
  msg,                          /* Pointer to picture data */
  NULL,                         /* Pointer to palette */
  GUI_DRAW_BMP24
};


//********************************************
static void CPU_CACHE_Enable(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
main(void)
{
  /* Enable the CPU Cache */
  CPU_CACHE_Enable();

  /* STM32F7xx HAL library initialization:
       - Configure the Flash ART accelerator on ITCM interface
       - Configure the Systick to generate an interrupt each 1 msec
       - Set NVIC Group Priority to 4
       - Global MSP (MCU Support Package) initialization
     */
  HAL_Init();

  /* Configure the system clock to 200 MHz */
  SystemClock_Config();

  /* Initialize LCD and LEDs */
  BSP_Config();
  

  /***********************************************************/

  /* Compute the prescaler value to have TIM3 counter clock equal to 10 KHz */
  uwPrescalerValue = (uint32_t) ((SystemCoreClock /2) / 10000) - 1;

  /* Set TIMx instance */
  TimHandle.Instance = TIM3;

  /* Initialize TIM3 peripheral as follows:
       + Period = 500 - 1
       + Prescaler = ((SystemCoreClock/2)/10000) - 1
       + ClockDivision = 0
       + Counter direction = Up
  */
 TimHandle.Init.Period = 750 - 1;
//   TimHandle.Init.Period = 50 - 1;
  TimHandle.Init.Prescaler = uwPrescalerValue;
  TimHandle.Init.ClockDivision = 0;
  TimHandle.Init.CounterMode = TIM_COUNTERMODE_UP;
  if(HAL_TIM_Base_Init(&TimHandle) != HAL_OK)
  {
    while(1)
    {
    }
  }

  /*##-2- Start the TIM Base generation in interrupt mode ####################*/
  /* Start Channel1 */
  if(HAL_TIM_Base_Start_IT(&TimHandle) != HAL_OK)
  {
    while(1)
    {
    }
  }

  /***********************************************************/

  /* Init the STemWin GUI Library */
  BSP_SDRAM_Init(); /* Initializes the SDRAM device */
  __HAL_RCC_CRC_CLK_ENABLE(); /* Enable the CRC Module */
  GUI_Init();

//   GUI_DispStringAt("Starting...", 0, 0);

  GUI_Initialized = 1;

  /* Activate the use of memory device feature */
  WM_SetCreateFlags(WM_CF_MEMDEV);

  /***********************************************************/
  /* Put the USART peripheral in the Asynchronous mode (UART Mode) */
  /* UART configured as follows:
      - Word Length = 8 Bits (7 data bit + 1 parity bit) : BE CAREFUL : Program 7 data bits + 1 parity bit in PC HyperTerminal
      - Stop Bit    = One Stop bit
      - Parity      = ODD parity
      - BaudRate    = 9600 baud
      - Hardware flow control disabled (RTS and CTS signals) */
  UartHandle.Instance        = USARTx;

  UartHandle.Init.BaudRate   = 115200;
  UartHandle.Init.WordLength = UART_WORDLENGTH_8B;
  UartHandle.Init.StopBits   = UART_STOPBITS_1;
  UartHandle.Init.Parity     = UART_PARITY_NONE;
  UartHandle.Init.HwFlowCtl  = UART_HWCONTROL_NONE;
  UartHandle.Init.Mode       = UART_MODE_TX_RX;

  if (HAL_UART_Init(&UartHandle) != HAL_OK)
  {
    while(1)
    {
    }
  }

  /***********************************************************/

  MainTask();


  //printf("!GRDY*\r\n");


  BSP_LED_On(LED1);



  /***********************************************************/

  /* Infinite loop */
  for(;;);
}

PUTCHAR_PROTOTYPE
{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the EVAL_COM1 and Loop until the end of transmission */
  HAL_UART_Transmit(&UartHandle, (uint8_t *)&ch, 1, 0xFFFF);

  return ch;
}


// void delay_ms(int t) {
//     int i;
//     while ( t-- > 0 ) {
// //         i = 200000;
//         i = 100000;
//         while ( i-- > 0 ) {}
//     };
// }





/**
  * @brief  Period elapsed callback in non blocking mode
  * @param  htim: TIM handle
  * @retval None
  */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
  BSP_Background();
}

/**
  * @brief TIM MSP Initialization
  *        This function configures the hardware resources used in this application:
  *           - Peripheral's clock enable
  *           - Peripheral's GPIO Configuration
  * @param htim: TIM handle pointer
  * @retval None
  */
void HAL_TIM_Base_MspInit(TIM_HandleTypeDef *htim)
{
  /*##-1- Enable peripherals and GPIO Clocks #################################*/
  /* TIMx Peripheral clock enable */
  __HAL_RCC_TIM3_CLK_ENABLE();

  /*##-2- Configure the NVIC for TIMx ########################################*/
  /* Set the TIMx priority */
  HAL_NVIC_SetPriority(TIM3_IRQn, 0, 1);

  /* Enable the TIMx global Interrupt */
  HAL_NVIC_EnableIRQ(TIM3_IRQn);
}

/**
  * @brief  Initializes the STM32756G-EVAL's LCD and LEDs resources.
  * @param  None
  * @retval None
  */
static void BSP_Config(void)
{
  /* Configure LED1, LED2, LED3 and LED4 */
//   BSP_LED_Init(LED1);
//   BSP_LED_Init(LED2);
//   BSP_LED_Init(LED3);
//   BSP_LED_Init(LED4);

  /* Initialize the Touch screen */
#ifdef USE_MB1046
  BSP_TS_Init(480, 272);
#elif defined(USE_MB1063)
  BSP_TS_Init(640, 480);
#endif
}

/**
* @brief  BSP_Background.
* @param  None
* @retval None
*/
void BSP_Background(void)
{
/*  BSP_LED_Toggle(LED1);
  BSP_LED_Toggle(LED2);
  BSP_LED_Toggle(LED3);
  BSP_LED_Toggle(LED4);
*/
  /* Capture input event and updade cursor */
  if(GUI_Initialized == 1)
  {
    BSP_Pointer_Update();
  }
}


/**
  * @brief  Provide the GUI with current state of the touch screen
  * @param  None
  * @retval None
  */
void BSP_Pointer_Update(void)
{
    TS_StateTypeDef  ts;

    ts.x = 0;
    ts.y = 0;
    ts.TouchDetected = 0;
    BSP_TS_GetState(&ts);

    HAL_StatusTypeDef status;
//     int i;
//     int j;

    btn_state = get_button(&ts, btns, BTN_CNT);

    if ( btn_state != btn_state_prev ) {


        btn_press   = ~btn_active &  btn_state;
        btn_release =  btn_active & ~btn_state;

        if ( btn_press & BTN_RST ) {
            btn_active |= BTN_RST;
            draw_button(&btns[0], BTN_ACTIVE, "Reinit");
            printf("!RST*\r\n");
        }
        if ( btn_release & BTN_RST ) {
            btn_active &= ~BTN_RST;
            draw_button(&btns[0], BTN_INACTIVE, "Reinit");
        }

        if ( btn_press & BTN_IMG ) {
            btn_active |= BTN_IMG;
            draw_button(&btns[1], BTN_ACTIVE, "Preview");
            printf("!GIMG*\r\n");
            
//             msg_active = 0;
//             msg_len = 0;
            msg_parse = 0;
            

            // HAL_UART_Receive can receive 2^16
            status = HAL_UART_Receive(&UartHandle, (uint8_t *)msg, 4, 0xFFFF);

            if ( (msg[0] == '#') && (msg[1] == 'I') && (msg[2] == 'M') && (msg[3] == 'G') ) {

                status = HAL_UART_Receive(&UartHandle, (uint8_t *)msg, ((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 2), 0xFFFF);
                status = HAL_UART_Receive(&UartHandle, (uint8_t *)(msg+(((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 2))), ((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 2) + 1, 0xFFFF);              

//                 for (i = 0; i < IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES; i++) {
//                     status = HAL_UART_Receive(&UartHandle, (uint8_t *)&msg[i], 1, 0xFFFF);                
//                 }


                if (status == HAL_OK) {
                    msg_parse = 1;              
                } 
            }
              
            

            // parse received message
            if ( msg_parse ) {
                          
                msg_parse = 0;                                            
                GUI_DrawBitmap(&_bm_img, 345+3, 170+3);

            }
            
        }
        if ( btn_release & BTN_IMG ) {
            btn_active &= ~BTN_IMG;
            draw_button(&btns[1], BTN_INACTIVE, "Preview");
        }


        btn_state_prev = btn_state;
    }


    sldr_v_state = get_slider_v(&ts, sldrs_v, SLDR_V_CNT);

    if ( sldr_v_state ) {

        if ( ( sldr_v_state & SLDR_V_AG ) && (sldrs_v[0].pos != sldrs_v[0].pos_prev) ) {
            draw_slider_v(&sldrs_v[0]);
            printf("!SAG%d*\r\n", sldrs_v[0].pos);
            sldrs_v[0].pos_prev = sldrs_v[0].pos;
        }

        if ( ( sldr_v_state & SLDR_V_DG ) && (sldrs_v[1].pos != sldrs_v[1].pos_prev) ) {
            draw_slider_v(&sldrs_v[1]);
            printf("!SDG%d*\r\n", sldrs_v[1].pos);
            sldrs_v[1].pos_prev = sldrs_v[1].pos;
        }

    }

}



/**
  * @brief  System Clock Configuration
  *         The system Clock is configured as follow :
  *            System Clock source            = PLL (HSE)
  *            SYSCLK(Hz)                     = 200000000
  *            HCLK(Hz)                       = 200000000
  *            AHB Prescaler                  = 1
  *            APB1 Prescaler                 = 4
  *            APB2 Prescaler                 = 2
  *            HSE Frequency(Hz)              = 25000000
  *            PLL_M                          = 25
  *            PLL_N                          = 400
  *            PLL_P                          = 2
  *            PLL_Q                          = 8
  *            VDD(V)                         = 3.3
  *            Main regulator output voltage  = Scale1 mode
  *            Flash Latency(WS)              = 6
  * @param  None
  * @retval None
  */
static void SystemClock_Config(void)
{
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitTypeDef RCC_OscInitStruct;
  HAL_StatusTypeDef ret = HAL_OK;

  /* Enable HSE Oscillator and activate PLL with HSE as source */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 400;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 8;

  ret = HAL_RCC_OscConfig(&RCC_OscInitStruct);
  if(ret != HAL_OK)
  {
    while(1) { ; }
  }

  /* Activate the OverDrive to reach the 200 MHz Frequency */
  ret = HAL_PWREx_EnableOverDrive();
  if(ret != HAL_OK)
  {
    while(1) { ; }
  }

  /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2 clocks dividers */
  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

  ret = HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_6);
  if(ret != HAL_OK)
  {
    while(1) { ; }
  }
}

/**
  * @brief  CPU L1-Cache enable.
  * @param  None
  * @retval None
  */
static void CPU_CACHE_Enable(void)
{
  /* Enable I-Cache */
  SCB_EnableICache();

  /* Enable D-Cache */
  SCB_EnableDCache();
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {}
}
#endif

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
